@@ -4,7 +4,7 @@
 #
 # SPDX-License-Identifier: Apache-2.0 OR GPL-2.0-or-later
 # Dulwich is dual-licensed under the Apache License, Version 2.0 and the GNU
-# General Public License as public by the Free Software Foundation; version 2.0
+# General Public License as published by the Free Software Foundation; version 2.0
 # or (at your option) any later version. You can redistribute it and/or
 # modify it under the terms of either of these two licenses.
 #
@@ -22,9 +22,11 @@
 
 """Generic functions for talking the git smart server protocol."""
 
+import types
+from collections.abc import Iterable, Sequence
 from io import BytesIO
 from os import SEEK_END
-from typing import Optional
+from typing import Callable, Optional
 
 import dulwich
 
@@ -128,30 +130,73 @@ DEPTH_INFINITE = 0x7FFFFFFF
 NAK_LINE = b"NAK\n"
 
 
-def agent_string():
+def agent_string() -> bytes:
+    """Generate the agent string for dulwich.
+
+    Returns:
+      Agent string as bytes
+    """
     return ("dulwich/" + ".".join(map(str, dulwich.__version__))).encode("ascii")
 
 
-def capability_agent():
+def capability_agent() -> bytes:
+    """Generate the agent capability string.
+
+    Returns:
+      Agent capability with dulwich version
+    """
     return CAPABILITY_AGENT + b"=" + agent_string()
 
 
-def capability_symref(from_ref, to_ref):
+def capability_symref(from_ref: bytes, to_ref: bytes) -> bytes:
+    """Generate a symref capability string.
+
+    Args:
+      from_ref: Source reference name
+      to_ref: Target reference name
+
+    Returns:
+      Symref capability string
+    """
     return CAPABILITY_SYMREF + b"=" + from_ref + b":" + to_ref
 
 
-def extract_capability_names(capabilities):
+def extract_capability_names(capabilities: Iterable[bytes]) -> set[bytes]:
+    """Extract capability names from a list of capabilities.
+
+    Args:
+      capabilities: List of capability strings
+
+    Returns:
+      Set of capability names
+    """
     return {parse_capability(c)[0] for c in capabilities}
 
 
-def parse_capability(capability):
+def parse_capability(capability: bytes) -> tuple[bytes, Optional[bytes]]:
+    """Parse a capability string into name and value.
+
+    Args:
+      capability: Capability string
+
+    Returns:
+      Tuple of (capability_name, capability_value)
+    """
     parts = capability.split(b"=", 1)
     if len(parts) == 1:
         return (parts[0], None)
-    return tuple(parts)
+    return (parts[0], parts[1])
 
 
-def symref_capabilities(symrefs):
+def symref_capabilities(symrefs: Iterable[tuple[bytes, bytes]]) -> list[bytes]:
+    """Generate symref capability strings from symref pairs.
+
+    Args:
+      symrefs: Iterable of (from_ref, to_ref) tuples
+
+    Returns:
+      List of symref capability strings
+    """
     return [capability_symref(*k) for k in symrefs]
 
 
@@ -163,18 +208,35 @@ COMMAND_WANT = b"want"
 COMMAND_HAVE = b"have"
 
 
-def format_cmd_pkt(cmd, *args):
+def format_cmd_pkt(cmd: bytes, *args: bytes) -> bytes:
+    """Format a command packet.
+
+    Args:
+      cmd: Command name
+      *args: Command arguments
+
+    Returns:
+      Formatted command packet
+    """
     return cmd + b" " + b"".join([(a + b"\0") for a in args])
 
 
-def parse_cmd_pkt(line):
+def parse_cmd_pkt(line: bytes) -> tuple[bytes, list[bytes]]:
+    """Parse a command packet.
+
+    Args:
+      line: Command line to parse
+
+    Returns:
+      Tuple of (command, [arguments])
+    """
     splice_at = line.find(b" ")
     cmd, args = line[:splice_at], line[splice_at + 1 :]
     assert args[-1:] == b"\x00"
     return cmd, args[:-1].split(b"\0")
 
 
-def pkt_line(data):
+def pkt_line(data: Optional[bytes]) -> bytes:
     """Wrap data in a pkt-line.
 
     Args:
@@ -187,7 +249,7 @@ def pkt_line(data):
     return ("%04x" % (len(data) + 4)).encode("ascii") + data
 
 
-def pkt_seq(*seq):
+def pkt_seq(*seq: Optional[bytes]) -> bytes:
     """Wrap a sequence of data in pkt-lines.
 
     Args:
@@ -196,16 +258,6 @@ def pkt_seq(*seq):
     return b"".join([pkt_line(s) for s in seq]) + pkt_line(None)
 
 
-def filter_ref_prefix(refs, prefixes):
-    """Filter refs to only include those with a given prefix.
-
-    Args:
-      refs: A list of refs.
-      prefixes: The prefixes to filter by.
-    """
-    return {k: v for k, v in refs.items() if any(k.startswith(p) for p in prefixes)}
-
-
 class Protocol:
     """Class for interacting with a remote git process over the wire.
 
@@ -218,7 +270,21 @@ class Protocol:
         Documentation/technical/protocol-common.txt
     """
 
-    def __init__(self, read, write, close=None, report_activity=None) -> None:
+    def __init__(
+        self,
+        read: Callable[[int], bytes],
+        write: Callable[[bytes], Optional[int]],
+        close: Optional[Callable[[], None]] = None,
+        report_activity: Optional[Callable[[int, str], None]] = None,
+    ) -> None:
+        """Initialize Protocol.
+
+        Args:
+          read: Function to read bytes from the transport
+          write: Function to write bytes to the transport
+          close: Optional function to close the transport
+          report_activity: Optional function to report activity
+        """
         self.read = read
         self.write = write
         self._close = close
@@ -226,16 +292,24 @@ class Protocol:
         self._readahead: Optional[BytesIO] = None
 
     def close(self) -> None:
+        """Close the underlying transport if a close function was provided."""
         if self._close:
             self._close()
 
-    def __enter__(self):
+    def __enter__(self) -> "Protocol":
+        """Enter context manager."""
         return self
 
-    def __exit__(self, exc_type, exc_val, exc_tb):
+    def __exit__(
+        self,
+        exc_type: Optional[type[BaseException]],
+        exc_val: Optional[BaseException],
+        exc_tb: Optional[types.TracebackType],
+    ) -> None:
+        """Exit context manager and close transport."""
         self.close()
 
-    def read_pkt_line(self):
+    def read_pkt_line(self) -> Optional[bytes]:
         """Reads a pkt-line from the remote git process.
 
         This method may read from the readahead buffer; see unread_pkt_line.
@@ -287,7 +361,7 @@ class Protocol:
         self.unread_pkt_line(next_line)
         return False
 
-    def unread_pkt_line(self, data) -> None:
+    def unread_pkt_line(self, data: Optional[bytes]) -> None:
         """Unread a single line of data into the readahead buffer.
 
         This method can be used to unread a single pkt-line into a fixed
@@ -303,7 +377,7 @@ class Protocol:
             raise ValueError("Attempted to unread multiple pkt-lines.")
         self._readahead = BytesIO(pkt_line(data))
 
-    def read_pkt_seq(self):
+    def read_pkt_seq(self) -> Iterable[bytes]:
         """Read a sequence of pkt-lines from the remote git process.
 
         Returns: Yields each line of data up to but not including the next
@@ -314,7 +388,7 @@ class Protocol:
             yield pkt
             pkt = self.read_pkt_line()
 
-    def write_pkt_line(self, line) -> None:
+    def write_pkt_line(self, line: Optional[bytes]) -> None:
         """Sends a pkt-line to the remote git process.
 
         Args:
@@ -329,7 +403,7 @@ class Protocol:
         except OSError as exc:
             raise GitProtocolError(str(exc)) from exc
 
-    def write_sideband(self, channel, blob) -> None:
+    def write_sideband(self, channel: int, blob: bytes) -> None:
         """Write multiplexed data to the sideband.
 
         Args:
@@ -343,7 +417,7 @@ class Protocol:
             self.write_pkt_line(bytes(bytearray([channel])) + blob[:65515])
             blob = blob[65515:]
 
-    def send_cmd(self, cmd, *args) -> None:
+    def send_cmd(self, cmd: bytes, *args: bytes) -> None:
         """Send a command and some arguments to a git server.
 
         Only used for the TCP git protocol (git://).
@@ -354,7 +428,7 @@ class Protocol:
         """
         self.write_pkt_line(format_cmd_pkt(cmd, *args))
 
-    def read_cmd(self):
+    def read_cmd(self) -> tuple[bytes, list[bytes]]:
         """Read a command and some arguments from the git client.
 
         Only used for the TCP git protocol (git://).
@@ -362,10 +436,12 @@ class Protocol:
         Returns: A tuple of (command, [list of arguments]).
         """
         line = self.read_pkt_line()
+        if line is None:
+            raise GitProtocolError("Expected command, got flush packet")
         return parse_cmd_pkt(line)
 
 
-_RBUFSIZE = 8192  # Default read buffer size.
+_RBUFSIZE = 65536  # 64KB buffer for better network I/O performance
 
 
 class ReceivableProtocol(Protocol):
@@ -381,14 +457,36 @@ class ReceivableProtocol(Protocol):
     """
 
     def __init__(
-        self, recv, write, close=None, report_activity=None, rbufsize=_RBUFSIZE
+        self,
+        recv: Callable[[int], bytes],
+        write: Callable[[bytes], Optional[int]],
+        close: Optional[Callable[[], None]] = None,
+        report_activity: Optional[Callable[[int, str], None]] = None,
+        rbufsize: int = _RBUFSIZE,
     ) -> None:
+        """Initialize ReceivableProtocol.
+
+        Args:
+          recv: Function to receive bytes from the transport
+          write: Function to write bytes to the transport
+          close: Optional function to close the transport
+          report_activity: Optional function to report activity
+          rbufsize: Read buffer size
+        """
         super().__init__(self.read, write, close=close, report_activity=report_activity)
         self._recv = recv
         self._rbuf = BytesIO()
         self._rbufsize = rbufsize
 
-    def read(self, size):
+    def read(self, size: int) -> bytes:
+        """Read bytes from the socket.
+
+        Args:
+          size: Number of bytes to read
+
+        Returns:
+          Bytes read from socket
+        """
         # From _fileobj.read in socket.py in the Python 2.6.5 standard library,
         # with the following modifications:
         #  - omit the size <= 0 branch
@@ -449,7 +547,15 @@ class ReceivableProtocol(Protocol):
         buf.seek(start)
         return buf.read()
 
-    def recv(self, size):
+    def recv(self, size: int) -> bytes:
+        """Receive bytes from the socket with buffering.
+
+        Args:
+          size: Maximum number of bytes to receive
+
+        Returns:
+          Bytes received from socket
+        """
         assert size > 0
 
         buf = self._rbuf
@@ -473,7 +579,7 @@ class ReceivableProtocol(Protocol):
         return buf.read(size)
 
 
-def extract_capabilities(text):
+def extract_capabilities(text: bytes) -> tuple[bytes, list[bytes]]:
     """Extract a capabilities list from a string, if present.
 
     Args:
@@ -486,7 +592,7 @@ def extract_capabilities(text):
     return (text, capabilities.strip().split(b" "))
 
 
-def extract_want_line_capabilities(text):
+def extract_want_line_capabilities(text: bytes) -> tuple[bytes, list[bytes]]:
     """Extract a capabilities list from a want line, if present.
 
     Note that want lines have capabilities separated from the rest of the line
@@ -504,7 +610,7 @@ def extract_want_line_capabilities(text):
     return (b" ".join(split_text[:2]), split_text[2:])
 
 
-def ack_type(capabilities):
+def ack_type(capabilities: Iterable[bytes]) -> int:
     """Extract the ack type from a capabilities list."""
     if b"multi_ack_detailed" in capabilities:
         return MULTI_ACK_DETAILED
@@ -521,7 +627,9 @@ class BufferedPktLineWriter:
     (including length prefix) reach the buffer size.
     """
 
-    def __init__(self, write, bufsize=65515) -> None:
+    def __init__(
+        self, write: Callable[[bytes], Optional[int]], bufsize: int = 65515
+    ) -> None:
         """Initialize the BufferedPktLineWriter.
 
         Args:
@@ -533,7 +641,7 @@ class BufferedPktLineWriter:
         self._wbuf = BytesIO()
         self._buflen = 0
 
-    def write(self, data) -> None:
+    def write(self, data: bytes) -> None:
         """Write data, wrapping it in a pkt-line."""
         line = pkt_line(data)
         line_len = len(line)
@@ -560,11 +668,16 @@ class BufferedPktLineWriter:
 class PktLineParser:
     """Packet line parser that hands completed packets off to a callback."""
 
-    def __init__(self, handle_pkt) -> None:
+    def __init__(self, handle_pkt: Callable[[Optional[bytes]], None]) -> None:
+        """Initialize PktLineParser.
+
+        Args:
+          handle_pkt: Callback function to handle completed packets
+        """
         self.handle_pkt = handle_pkt
         self._readahead = BytesIO()
 
-    def parse(self, data) -> None:
+    def parse(self, data: bytes) -> None:
         """Parse a fragment of data and call back for any completed packets."""
         self._readahead.write(data)
         buf = self._readahead.getvalue()
@@ -583,31 +696,76 @@ class PktLineParser:
         self._readahead = BytesIO()
         self._readahead.write(buf)
 
-    def get_tail(self):
+    def get_tail(self) -> bytes:
         """Read back any unused data."""
         return self._readahead.getvalue()
 
 
-def format_capability_line(capabilities):
+def format_capability_line(capabilities: Iterable[bytes]) -> bytes:
+    """Format a capabilities list for the wire protocol.
+
+    Args:
+      capabilities: List of capability strings
+
+    Returns:
+      Space-separated capabilities as bytes
+    """
     return b"".join([b" " + c for c in capabilities])
 
 
-def format_ref_line(ref, sha, capabilities=None):
+def format_ref_line(
+    ref: bytes, sha: bytes, capabilities: Optional[Sequence[bytes]] = None
+) -> bytes:
+    """Format a ref advertisement line.
+
+    Args:
+      ref: Reference name
+      sha: SHA hash
+      capabilities: Optional list of capabilities
+
+    Returns:
+      Formatted ref line
+    """
     if capabilities is None:
         return sha + b" " + ref + b"\n"
     else:
         return sha + b" " + ref + b"\0" + format_capability_line(capabilities) + b"\n"
 
 
-def format_shallow_line(sha):
+def format_shallow_line(sha: bytes) -> bytes:
+    """Format a shallow line.
+
+    Args:
+      sha: SHA to mark as shallow
+
+    Returns:
+      Formatted shallow line
+    """
     return COMMAND_SHALLOW + b" " + sha
 
 
-def format_unshallow_line(sha):
+def format_unshallow_line(sha: bytes) -> bytes:
+    """Format an unshallow line.
+
+    Args:
+      sha: SHA to unshallow
+
+    Returns:
+      Formatted unshallow line
+    """
     return COMMAND_UNSHALLOW + b" " + sha
 
 
-def format_ack_line(sha, ack_type=b""):
+def format_ack_line(sha: bytes, ack_type: bytes = b"") -> bytes:
+    """Format an ACK line.
+
+    Args:
+      sha: SHA to acknowledge
+      ack_type: Optional ACK type (e.g. b"continue")
+
+    Returns:
+      Formatted ACK line
+    """
     if ack_type:
         ack_type = b" " + ack_type
     return b"ACK " + sha + ack_type + b"\n"
